#!/usr/bin/env python
# coding:utf-8
import collections
from WeekAlarmUI import WeekAlarmUI
from operator import attrgetter
import ConfigParser
from logging import debug,info,exception,error,warning, handlers
import os
import subprocess
from amarok import Amarok,DcopAmaroK
PL_NO = 0
PL_SAVE = 1
PL_FILE = 2
PL_DYNAMIC = 3
VOL_NORMAL = 0
VOL_DECREASE = 1
VOL_INCREASE = 2
def importerror(message, e = None):
    subprocess.call(["kdialog", "--sorry", message])
    if e:
        raise e
try:
    import qt
except ImportError,e:
    importerror("Sorry, you need PyQT binding",e)
days = ["mon","tue","wed","thu","fri","sat","sun"]

class Config(dict):
    class MissingOptions(Exception):
        def __init__(self,l):
            Exception.__init__(self,"%s options are missing"%len(l))
            self.missing = l
    def __init__(self,path,configdict):
        dict.__init__(self)
        self.path = path
        self._parsedict(configdict)
    def _parsedict(self,d):
        self.configdict = d
        self.widgets = {} # qt interface (getter,setter)
        self.shorts = {} # weekalarm interface
        self.configitems = {} # file iterface (from and to string)
        for widgetType, values in self.configdict.items():
            widgets = values[0]
            funcs = list(values[1:])
            # if funcs is too short, fill it in with def values
            defs = [None,None,str,str]
            for i in range(len(funcs),len(defs)):
                if defs[i] is None:
                    raise ValueError("arg %i mandatory"%i)
                funcs.append(defs[i])
            for widget, (sect, param, short) in widgets.items():
                self.widgets[(sect,param)] = (widget,) + tuple(funcs[:2])
                self.configitems[(sect, param)] = tuple(
                                        funcs[2:]
                                        )
                self.shorts[short] = (sect,param)

    def read(self,path = None):
        """file -> config"""
        if path is None:
                path = self.path
        debug("load config at %s"%path)
        config = ConfigParser.ConfigParser()
        config.read(self.path)
        for (sect,param),(from_string,to_string) in self.configitems.items():
            self[sect][param] = from_string(config.get(sect,param))

    def write(self,file=None):
        """config -> file"""
        config = ConfigParser.ConfigParser()
        for (sect,param),(from_string,to_string) in self.configitems.items():
                try:
                    config.add_section(sect)
                except ConfigParser.DuplicateSectionError:
                    pass
                config.set(sect,param,to_string(self[sect][param]))
        if not file:
            file = open(self.path,"w")
        debug("write config to %s"%file.name)
        config.write(file)

    def __getitem__(self,key):
        s = self.shorts
        if key in s:
            return self[s[key][0]][s[key][1]]
        return self.setdefault(key, dict())

    def __setitem__(self,key,attr):
        s = self.shorts
        if key in s:
            self[s[key][0]][s[key][1]] = attr
        return super(Config, self).__setitem__(key, attr)


class ConfigDialog( WeekAlarmUI ):
    """ Configuration window """

    def __init__( self ):
        debug("creating config dialog")
        WeekAlarmUI.__init__( self )
        self.accepted = qt.PYSIGNAL("accepted()")
        self.initconfigitems()
        self.connectSlots()
        try:
            self.config.read()
            self.loadSettings()
        except ConfigParser.Error:
            warning("Reading configuration has failed, using default")
            self.saveSettings()
        self.update()

    def run(self):
        try:
            self.loadSettings()
        except Exception:
            pass
        self.update()
        self.show()
        self.setActiveWindow()
        self.raiseW()

    def loadSettings(self):
        """config -> window"""
        debug("loading settings")
        for (section, param),(widget, getter, setter) in\
            self.config.widgets.items():
           setter(widget)(self.config[section][param])

    def saveSettings(self):
        """window -> config"""
        debug("saving settings")
        for (sect, param), (widget, getter, setter) in\
            self.config.widgets.items():
                self.config[sect][param] = getter(widget)()
        self.config.write()


    def initconfigitems(self):
        #keys: widget
        #values: category and params in config file
        ag = attrgetter
        def bool_(string):
            return bool(eval(string))
        def fromutf8(s):
            debug("f:%r -> %r"%(s, s.decode("utf8")))
            return s.decode("utf8")
        def toutf8(u):
            debug("t:%r -> %r"%(u, u.encode("utf8")))
            return str(u.encode("utf8"))

        self.configdict = {
                qt.QCheckBox:(
                    {
                        self.napChk:("Nap", "enabled", "napMode"),
                        self.rndStartupChk:("Playlist", "random",
                            "random"),
                        self.rptStartupChk:("Playlist", "repeat",
                            "repeat"),
                        self.snoozeChk:("Snooze", "enabled",
                            "snooze"),
                        self.updateVolChk:("Alarm", "update_volume",
                            "updateVol"),
                        self.updateEndChk:("Alarm", "update_end_value",
                            "updateEnd"),
                        self.stopPauseChk:("Stop", "stop_pause",
                            "stopOnPause"),
                        self.volSensChk:("Stop", "volume_change",
                            "stopVolume"),
                        },
                    ag("isChecked"),#getter
                    ag("setChecked"),#setter
                    bool_, #from string def:str
                    str,  #to string def:str
                    ),
                qt.QSpinBox:(
                    {
                        self.napLenSel:("Nap", "length", "napLen"),
                        self.snoozeLenSel:("Snooze","length", "snoozeLen"),
                        self.startSel:("Alarm", "startVal", "fadeStart"),
                        self.endSel:("Alarm", "endVal", "fadeEnd"),
                        self.fadeSel:("Alarm", "fadeVal", "fadeTime"),
                        self.volSensSel:("Stop",
                        "volume_change_value", "VolumeSens"),
                        },
                    ag("value"),
                    ag("setValue"),
                    int,
                    ),
                qt.QButtonGroup:(
                    {
                        self.plGroup:("Playlist", "mode", "plMode"),
                        self.modeGroup:("Alarm", "mode", "volMode"),
                        },
                    ag("selectedId"),
                    ag("setButton"),
                    int,
                    ),
                qt.QTimeEdit:(
                    {},
                    ag("time"),
                    ag("setTime"),
                    qt.QTime.fromString,
                    qt.QTime.toString,
                    ),
                "playlist":(
                    {
                        self.plTxt:("Playlist", "playlist", "pl"),
                        },
                    lambda w:self.findPL,
                    ag("setText"),
                    fromutf8,
                    toutf8,

                    )
                }

# for each day, we have a check and a time selector
        for day in days:
            chk = getattr(self,day+"Chk")
            self.configdict[qt.QCheckBox][0][chk] = ("Days", day, day)
            sel = getattr(self,day+"TimeSel")
            self.configdict[qt.QTimeEdit][0][sel] = ("Time", day, day + "Time")

        self.config = Config("weekalarmrc", self.configdict)
        #The key represent a qt object wich need to be inactived
        #if the function return false
        from pprint import pprint
        debug(pprint(dict(self.config)))
        self.depends = {
                self.dailyGroup: lambda: not self.napChk.isChecked(),
                self.napLenSel: self.napChk.isChecked,
                self.napLbl: self.napChk.isChecked,
                self.plPick: lambda: self.plGroup.selectedId() == PL_FILE,
                self.plLbl:  lambda: self.plGroup.selectedId() >= PL_FILE,
                self.plTxt:  lambda: self.plGroup.selectedId() >= PL_FILE,
                self.snoozeLbl: self.snoozeChk.isChecked,
                self.snoozeLenSel: self.snoozeChk.isChecked,
                # page "fading" only if mode is not "just start"
                self.tabWidget.page(2):
                    lambda: self.modeGroup.selectedId() > 0,
                self.volBox: lambda: self.modeGroup.selectedId() ==
                    VOL_INCREASE,
                self.updateEndChk: self.updateVolChk.isChecked,
        }

    def connectSlots(self):
        self.connect(self.startSel, qt.SIGNAL("valueChanged(int)"),self.endSel.setMinValue)
        self.connect(self.cancel, qt.SIGNAL("clicked()"),self.on_cancel_clicked)
        self.connect(self.save, qt.SIGNAL("clicked()"),self.on_save_clicked)
        self.connect(self.fill, qt.SIGNAL("clicked()"),self.on_fill_clicked)
        self.connect(self.startSel, qt.SIGNAL("valueChanged(int)"),self.endSel.setMinValue)
        self.connect(self.plPick, qt.SIGNAL("clicked()"),self.on_plPick_clicked)

        for (w, _, _) in self.config.widgets.values():
            if isinstance(w, qt.QCheckBox):
                self.connect(w, qt.SIGNAL("clicked()"), self.update) # checks
            if isinstance(w, qt.QButtonGroup):
                self.connect(w, qt.SIGNAL("clicked(int)"), lambda a:self.update())

    def on_save_clicked(self):
        self.config.write()
        self.accept()

    def on_cancel_clicked(self):
        self.reject()

    def on_fill_clicked(self):
        time = self.monTimeSel.time()
        for day in days[1:]:
            getattr(self , day + "TimeSel").setTime(time)

    def update(self):
        debug("update")
        for widget, func in self.depends.items():
            widget.setEnabled(func())

    def on_plPick_clicked(self):
        self.pl = qt.QFileDialog.getOpenFileName("/home","Playlists (*.m3u *.xml)", self,"open playlist dialog","Select Custom Playlist")
        self.plTxt.setText(self.pl)

    def currPL_(self):
        plfile = DcopAmaroK().send("playlist", "saveCurrentPlaylist")
        os.rename(plfile,"weekalarmpl.xml")
        return os.path.join(os.getcwd(),"weekalarmpl.xml")

    def findPL(self):
        id = self.plGroup.selectedId()
        if id == PL_NO:
            return u"null"
        elif id == PL_SAVE:
            return unicode(self.currPL_())
        else: # PL_FILE or PL_DYNAMIC
            return unicode(self.plTxt.text())
        return "null"

    def accept(self):
        self.saveSettings()
        self.emit(self.accepted,())
        qt.QDialog.accept(self)

    def tr(self,s,c=None):
        return unicode(_(s),'utf-8')

if __name__ == '__main__':
    import sys
    def _(s):
        return s
    def bool_(s):
            return bool(eval(s))
    widgets = {
            "QCheckBox":(
            {
            "napChk":("Nap","enabled","napMode"), #widget:(sect,param,shortname)
            "snoozeChk":("Snooze","enabled","snooze"),
            },
        attrgetter("isChecked"),
        attrgetter("setChecked"),
        bool_, #from string
        str,  #to string
        ),
    }
    app = qt.QApplication(sys.argv)
    c = Config("weekalarmrc",widgets)
    cd = ConfigDialog()
    cd.exec_loop()
